@@ -2,8 +2,9 @@ package Simo;
 use strict;
 use warnings;
 use Carp;
+use Simo::Error;
 
-our $VERSION = '0.07_05';
+our $VERSION = '0.09_01';
 
 my %VALID_IMPORT_OPT = map{ $_ => 1 } qw( base mixin );
 sub import{
@@ -76,15 +77,27 @@ sub new{
     
     # set args
     while( my ( $attr, $val ) = splice( @args, 0, 2 ) ){
-        croak "Invalid key '$attr' is passed to new" unless $self->can( $attr );
-        
+        unless( $self->can( $attr ) ){
+            Simo::Error->throw(
+                type => 'attr_not_exist',
+                msg => "Invalid key '$attr' is passed to new",
+                pkg => $pkg,
+                attr => $attr
+            );
+        }
         no strict 'refs';
         $self->$attr( $val );
     }
     
     foreach my $required_attrs ( $self->REQUIRED_ATTRS ){
-        croak "Attr '$required_attrs' is required." 
-            unless exists $self->{ $required_attrs };
+        unless( exists $self->{ $required_attrs } ){
+            Simo::Error->throw(
+                type => 'attr_required',
+                msg => "Attr '$required_attrs' is required.",
+                pkg => $pkg,
+                attr => $required_attrs
+            );
+        }
     }
     return $self;
 }
@@ -92,90 +105,6 @@ sub new{
 # required keys when object is created by new.
 sub REQUIRED_ATTRS{ () }
 
-# get value specify attr names
-sub get_attrs{
-    my ( $self, @attrs ) = @_;
-    
-    @attrs = @{ $attrs[0] } if ref $attrs[0] eq 'ARRAY';
-    
-    my @vals;
-    foreach my $attr ( @attrs ){
-        croak "Invalid key '$attr' is passed to get_attrs" unless $self->can( $attr );
-        my $val = $self->$attr;
-        push @vals, $val;
-    }
-    wantarray ? @vals : $vals[0];
-}
-
-# get value as hash specify attr names
-sub get_attrs_as_hash{
-    my ( $self, @attrs ) = @_;
-    my @vals = $self->get_attrs( @attrs );
-    
-    my %attrs;
-    @attrs{ @attrs } = @vals;
-    
-    wantarray ? %attrs : \%attrs;
-}
-
-# set values
-sub set_attrs{
-    my ( $self, @args ) = @_;
-
-    # check args
-    @args = %{ $args[0] } if ref $args[0] eq 'HASH';
-    croak 'key-value pairs must be passed to set_attrs' if @args % 2;
-    
-    # set args
-    while( my ( $attr, $val ) = splice( @args, 0, 2 ) ){
-        croak "Invalid key '$attr' is passed to set_attrs" unless $self->can( $attr );
-        no strict 'refs';
-        $self->$attr( $val );
-    }
-    return $self;
-}
-
-# run methods
-sub run_methods{
-    my ( $self, @method_or_args_list ) = @_;
-    
-    my $method_infos = $self->_SIMO_parse_run_methods_args( @method_or_args_list );
-    while( my $method_info = shift @{ $method_infos } ){
-        my ( $method, $args ) = @{ $method_info }{ qw( name args ) };
-        
-        if( @{ $method_infos } ){
-            $self->$method( @{ $args } );
-        }
-        else{
-            return wantarray ? ( $self->$method( @{ $args } ) ) :
-                                 $self->$method( @{ $args } );
-        }
-    }
-}
-
-sub _SIMO_parse_run_methods_args{
-    my ( $self, @method_or_args_list ) = @_;
-    
-    my $method_infos = [];
-    while( my $method_or_args = shift @method_or_args_list ){
-        croak "$method_or_args is bad. Method name must be string and args must be array ref"
-            if ref $method_or_args;
-        
-        my $method = $method_or_args;
-        croak "$method is not exist" unless $self->can( $method );
-        
-        my $method_info = {};
-        $method_info->{ name } = $method;
-        $method_info->{ args } = ref $method_or_args_list[0] eq 'ARRAY' ?
-                                 shift @method_or_args_list :
-                                 [];
-        
-        push @{ $method_infos }, $method_info;
-    }
-    return $method_infos;
-}
-
-
 # create accessor
 sub ac(@){
     # Simo process
@@ -247,12 +176,23 @@ sub _SIMO_create_accessor{
         qq/{\n/ .
         # arg recieve
         qq/    my ( \$self, \@vals ) = \@_;\n\n/;
-    
+
     if( defined $ac_opt->{ default } ){
         # default value
         $e .=
-        qq/    if( ! exists( \$self->{ $attr } ) ){\n/ .
-        qq/        \$self->{ $attr } = \$ac_opt->{ default };\n/ .
+        qq/    if( ! exists( \$self->{ $attr } ) ){\n/;
+
+        if( ref $ac_opt->{ default } ){
+        $e .=
+        qq/        require Storable;\n/ .
+        qq/        \$self->{ $attr } = Storable::dclone( \$ac_opt->{ default } );\n/;
+        }
+        else{
+        $e .=
+        qq/        \$self->{ $attr } = \$ac_opt->{ default };\n/;
+        }
+        
+        $e .=
         qq/    }\n/ .
         qq/    \n/;
     }
@@ -305,7 +245,13 @@ sub _SIMO_create_accessor{
         qq/            my \$ret = \$constrain->( \$val );\n/ .
         qq/            if( !\$ret ){\n/ .
         qq/                \$@ ||= 'must be valid value.';\n/ .
-        qq/                Carp::croak( "${pkg}::$attr \$@" )\n/ .
+        qq/                Simo::Error->throw(\n/ .
+        qq/                    type => 'type_invalid',\n/ .
+        qq/                    msg => "${pkg}::$attr \$@",\n/ .
+        qq/                    pkg => "$pkg",\n/ .
+        qq/                    attr => "$attr",\n/ .
+        qq/                    val => \$val,\n/ .
+        qq/                );\n/ .
         qq/            }\n/ .
         qq/        }\n\n/;
     }
@@ -381,13 +327,117 @@ sub _SIMO_get_ac_info {
     return ( $self, $attr, $pkg, @vals );
 }
 
+###---------------------------------------------------------------------------
+# The following methods is not recommended function 
+# These method is not essential as nature of object.
+# To provide the same fanctionality, I create Simo::Wrapper.
+# See Also Simo::Wrapper
+# These methods will be removed in future 2019/01/01
+###---------------------------------------------------------------------------
+
+# get value specify attr names
+# ( not recommended )
+sub get_attrs{
+    my ( $self, @attrs ) = @_;
+    
+    @attrs = @{ $attrs[0] } if ref $attrs[0] eq 'ARRAY';
+    
+    my @vals;
+    foreach my $attr ( @attrs ){
+        croak "Invalid key '$attr' is passed to get_attrs" unless $self->can( $attr );
+        my $val = $self->$attr;
+        push @vals, $val;
+    }
+    wantarray ? @vals : $vals[0];
+}
+
+# get value as hash specify attr names
+# ( not recommended )
+sub get_attrs_as_hash{
+    my ( $self, @attrs ) = @_;
+    my @vals = $self->get_attrs( @attrs );
+    
+    my %attrs;
+    @attrs{ @attrs } = @vals;
+    
+    wantarray ? %attrs : \%attrs;
+}
+
+# set values
+# ( not recommended )
+sub set_attrs{
+    my ( $self, @args ) = @_;
+
+    # check args
+    @args = %{ $args[0] } if ref $args[0] eq 'HASH';
+    croak 'key-value pairs must be passed to set_attrs' if @args % 2;
+    
+    # set args
+    while( my ( $attr, $val ) = splice( @args, 0, 2 ) ){
+        croak "Invalid key '$attr' is passed to set_attrs" unless $self->can( $attr );
+        no strict 'refs';
+        $self->$attr( $val );
+    }
+    return $self;
+}
+
+# run methods
+# ( not recommended )
+sub run_methods{
+    my ( $self, @method_or_args_list ) = @_;
+    
+    my $method_infos = $self->_SIMO_parse_run_methods_args( @method_or_args_list );
+    while( my $method_info = shift @{ $method_infos } ){
+        my ( $method, $args ) = @{ $method_info }{ qw( name args ) };
+        
+        if( @{ $method_infos } ){
+            $self->$method( @{ $args } );
+        }
+        else{
+            return wantarray ? ( $self->$method( @{ $args } ) ) :
+                                 $self->$method( @{ $args } );
+        }
+    }
+}
+
+# ( not recommended )
+sub _SIMO_parse_run_methods_args{
+    my ( $self, @method_or_args_list ) = @_;
+    
+    my $method_infos = [];
+    while( my $method_or_args = shift @method_or_args_list ){
+        croak "$method_or_args is bad. Method name must be string and args must be array ref"
+            if ref $method_or_args;
+        
+        my $method = $method_or_args;
+        croak "$method is not exist" unless $self->can( $method );
+        
+        my $method_info = {};
+        $method_info->{ name } = $method;
+        $method_info->{ args } = ref $method_or_args_list[0] eq 'ARRAY' ?
+                                 shift @method_or_args_list :
+                                 [];
+        
+        push @{ $method_infos }, $method_info;
+    }
+    return $method_infos;
+}
+
 =head1 NAME
 
 Simo - Very simple framework for Object Oriented Perl.
 
 =head1 VERSION
 
-Version 0.07_05
+Version 0.09_01
+
+=cut
+
+=head1 CAUTION
+
+Simo is yet experimenta stage.
+
+Please wait until Simo will be stable.
 
 =cut
 
@@ -666,47 +716,26 @@ You can get old value when you use accessor as setter.
 
 =head2 ac
 
-ac is exported. This is used by define accessor. 
+ac is exported. This is used to define accessor.
 
-=head1 METHOD
+    package Book;
+    use Simo;
+    
+    sub title{ ac }
+    sub author{ ac }
+
+=head1 METHODS
 
 =head2 new
 
-orveridable new method.
-
-=head2 get_attrs
-
-    my( $title, $author ) = $book->get_attrs( 'title', 'author' );
-
-=head2  get_attrs_as_hash
-
-    my %hash = $book->get_attrs( 'title', 'author' );
+Orveridable new method. You can create object.
 
+    use Book;
+    my $book = Book->new( title => 'a', author => 'b' );
+    
 or
 
-    my $hash_ref = $book->get_attrs( 'title', 'author' );
-
-=head2 set_attrs
-
-    $book->set_attrs( title => 'Simple OO', author => 'kimoto' );
-
-return value is $self. so method chaine is available
-
-    $book->set_attrs( title => 'Simple OO', author => 'kimoto' )->some_method;
-    
-=head2 run_methods
-
-this excute some methods continuously.
-
-    my $result = $book_list->run_methods(
-        'select' => [ type => 'Commic' ],
-        'sort' => [ 'desc' },
-        'get_result'
-    );
-    
-args must be array ref. You can omit args.
-
-You can get last method return value in scalar context or list context.
+    my $book = Book->new( { title => 'a', author => 'b' } );
 
 =head2 REQUIRED_ATTRS
 
@@ -718,6 +747,44 @@ You can set required attrs when object is create by new.
 
 Please be careful not to write wrong spell.
 
+=head2 get_attrs
+
+This methods is not now recommended, Please do not use this method.
+
+I realize that this method is not essential for object, 
+and become to beleve this method is helper method for object.
+
+This method will be removed in future 2019/1/1.
+
+I implement the same functionality in L<Simo::Wrapper>.
+and this is used by L<Simo::Util> o function. 
+
+Pleae see L<Simo::Util> o function.
+    
+=head2  get_attrs_as_hash
+
+This methods is not now recommended for the same reason as get_attrs.
+
+This method will be removed in future 2019/1/1.
+
+Pleae see L<Simo::Util> o function.
+
+=head2 set_attrs
+
+This methods is not now recommended for the same reason as get_attrs.
+
+This method will be removed in future 2019/1/1.
+
+Pleae see L<Simo::Util> o function.
+
+=head2 run_methods
+
+This methods is not now recommended for the same reason as get_attrs.
+
+This method will be removed in future 2019/1/1.
+
+Pleae see L<Simo::Util> o function.
+
 =head1 MORE TECHNIQUES
 
 I teach you useful techniques.
